
function LevelLoadingScreenGuiScript:init(scene_gui, res, progress, base_layer)
	self._scene_gui = scene_gui
	self._res = res
	self._base_layer = base_layer
	self._level_tweak_data = arg.load_level_data.level_tweak_data
	self._gui_tweak_data = arg.load_level_data.gui_tweak_data
	self._menu_tweak_data = arg.load_level_data.menu_tweak_data
	self._scale_tweak_data = arg.load_level_data.scale_tweak_data
	self._coords = arg.load_level_data.controller_coords or false
	self._coords_font = arg.load_level_data.coords_font or "fonts/font_medium_mf"
	self._coords_font_size = arg.load_level_data.coords_font_size or 24
	self._gui_data = arg.load_level_data.gui_data
	self._workspace_size = self._gui_data.workspace_size
	self._saferect_size = self._gui_data.saferect_size
	local challenges = arg.load_level_data.challenges
	local safe_rect_pixels = self._gui_data.safe_rect_pixels
	local safe_rect = self._gui_data.safe_rect
	local aspect_ratio = self._gui_data.aspect_ratio
	self._safe_rect_pixels = safe_rect_pixels
	self._safe_rect = safe_rect
	self._gui_data_manager = GuiDataManager:new(self._scene_gui, res, safe_rect_pixels, safe_rect, aspect_ratio)
	self._back_drop_gui = MenuBackdropGUI:new(nil, self._gui_data_manager, true)
	self._back_drop_gui:set_pattern("guis/textures/loading/loading_foreground", 1)
	local base_panel = self._back_drop_gui:get_new_base_layer()
	local level_image = base_panel:bitmap({
		texture = self._gui_data.bg_texture,
		layer = 0
	})
	level_image:set_alpha(0.5)
	level_image:set_size(level_image:parent():h() * (level_image:texture_width() / level_image:texture_height()), level_image:parent():h())
	level_image:set_position(0, 0)
	local background_fullpanel = self._back_drop_gui:get_new_background_layer()
	local background_safepanel = self._back_drop_gui:get_new_background_layer()
	self._back_drop_gui:set_panel_to_saferect(background_safepanel)
	self._indicator = background_safepanel:bitmap({
		name = "indicator",
		texture = "guis/textures/icon_loading",
		layer = 0
	})
	self._level_title_text = background_safepanel:text({
		y = 0,
		text_id = "debug_loading_level",
		font = "fonts/font_large_mf",
		font_size = 36,
		color = Color.white,
		align = "left",
		halign = "left",
		vertical = "bottom",
		layer = 0,
		h = 36
	})
	self._level_title_text:set_text(utf8.to_upper(self._level_title_text:text()))
	local _, _, w, h = self._level_title_text:text_rect()
	self._level_title_text:set_size(w, h)
	self._indicator:set_right(self._indicator:parent():w())
	self._level_title_text:set_right(self._indicator:left())
	local bg_loading_text = background_fullpanel:text({
		y = 0,
		text_id = "debug_loading_level",
		font = "fonts/font_eroded",
		font_size = 80,
		h = 80,
		color = Color(0.3, 0.38039216, 0.8392157, 1),
		align = "right",
		vertical = "top",
		layer = 0
	})
	bg_loading_text:set_text(utf8.to_upper(bg_loading_text:text()))
	local x, y = self._level_title_text:world_right(), self._level_title_text:world_center_y()
	bg_loading_text:set_world_right(x)
	bg_loading_text:set_world_center_y(y)
	bg_loading_text:move(13, 3)
	self._back_drop_gui:animate_bg_text(bg_loading_text)
	if self._coords then
		self._controller_shapes = arg.load_level_data.controller_shapes or {
			{
				position = {cx = 0.5, cy = 0.5},
				texture_rect = {
					0,
					0,
					512,
					256
				}
			}
		}
		self._controllers = {}
		local controller, position
		for i, shape in ipairs(self._controller_shapes) do
			controller = background_safepanel:bitmap({
				texture = arg.load_level_data.controller_image or "guis/textures/controller",
				layer = i,
				texture_rect = shape.texture_rect
			})
			position = shape.position or {}
			controller:set_center(background_safepanel:w() * (position.cx or 0.5), background_safepanel:h() * (position.cy or 0.5))
			if position.x then
				if 0 > position.x then
					controller:set_right(background_safepanel:w() + position.x)
				else
					controller:set_left(position.x)
				end
			end
			if position.y then
				if 0 > position.y then
					controller:set_bottom(background_safepanel:h() + position.y)
				else
					controller:set_top(position.y)
				end
			end
			controller:move(position.mx or 0, position.my or 0)
			table.insert(self._controllers, controller)
		end
		for id, data in pairs(self._coords) do
			controller = self._controllers[data.c or 1]
			data.text = background_safepanel:text({
				name = data.id,
				text = data.string,
				font_size = self._coords_font_size,
				font = self._coords_font,
				align = data.align,
				vertical = data.vertical,
				halign = "center",
				valign = "center",
				color = data.color
			})
			local _, _, w, h = data.text:text_rect()
			data.text:set_size(w, h)
			if data.x then
				local x = controller:x() + data.x
				local y = controller:y() + data.y
				if data.align == "left" then
					data.text:set_left(x)
				elseif data.align == "right" then
					data.text:set_right(x)
				elseif data.align == "center" then
					data.text:set_center_x(x)
				end
				if data.vertical == "top" then
					data.text:set_top(y)
				elseif data.vertical == "bottom" then
					data.text:set_bottom(y)
				else
					data.text:set_center_y(y)
				end
			end
		end
	end
end
